@@ -39,7 +39,7 @@ static snd_output_t *output = NULL;
 
 
 
-void print_state() 
+void print_state(void)
 {
   
 	int state;
@@ -93,14 +93,15 @@ snd_pcm_t* initAlsa(char *dev, int format, int channels, int speed, int mode)
 	snd_pcm_t *handle;
 	snd_pcm_hw_params_t *hwparams;
 	int err;
-	int periods;
+	unsigned int periods;
   
 #ifdef DRIVER_ALSA_09_NEW_PCM_API
 	int t_dir=0;
-	int t_speed=speed;
+	unsigned int t_speed=(unsigned int)speed;
 	snd_pcm_uframes_t t_bufsize=BUFFERSIZE;
 #endif
 
+	snd_config_update_free_global();
 	err = snd_pcm_open(&handle, dev, mode, SND_PCM_NONBLOCK);
 	if (err < 0) {
 		if (alsadbg)
@@ -173,7 +174,7 @@ snd_pcm_t* initAlsa(char *dev, int format, int channels, int speed, int mode)
 	}
   
 	periods = 2;
-	err = snd_pcm_hw_params_set_periods_min(handle, hwparams, &periods, 0);
+	err = snd_pcm_hw_params_set_periods_min(handle, hwparams, &periods, NULL);
 	if (err < 0) {
 		if (alsadbg)
 			fprintf(stderr, "%s\n", snd_strerror(err));
@@ -182,7 +183,7 @@ snd_pcm_t* initAlsa(char *dev, int format, int channels, int speed, int mode)
 	}
   
 	periods = 64;
-	err = snd_pcm_hw_params_set_periods_max(handle, hwparams, &periods, 0);
+	err = snd_pcm_hw_params_set_periods_max(handle, hwparams, &periods, NULL);
 	if (err < 0) {
 		if (alsadbg)
 			fprintf(stderr, "%s\n", snd_strerror(err));
@@ -215,10 +216,10 @@ snd_pcm_t* initAlsa(char *dev, int format, int channels, int speed, int mode)
 
 #define ARCH_esd_audio_devices
 
-const char *esd_audio_devices()
+const char *esd_audio_devices(void)
 {
         int card, err;
-        static char *all_alsa_cards=0;
+        static char *all_alsa_cards = NULL;
         char *alsa_card_tmp;
         snd_ctl_t *handle;
         snd_ctl_card_info_t *info;
@@ -228,7 +229,7 @@ const char *esd_audio_devices()
         if(all_alsa_cards)
         {
                 free(all_alsa_cards);
-                all_alsa_cards = 0;
+                all_alsa_cards = NULL;
         }
 
 	card = -1;
@@ -288,12 +289,11 @@ const char *esd_audio_devices()
 
 #define ARCH_esd_audio_open
 
-int esd_audio_open()
+int esd_audio_open(void)
 {
   
 	int channels;
 	int format;
-	int card;
 	char *dev;
   
 	if (alsadbg)
@@ -327,7 +327,7 @@ int esd_audio_open()
 		fprintf(stderr, "dev=%s\n",dev);
 
 	alsa_playback_handle = initAlsa(dev, format, channels, 
-									esd_audio_rate, SND_PCM_STREAM_PLAYBACK);
+				esd_audio_rate, SND_PCM_STREAM_PLAYBACK);
 	if(alsaerr) 
 	{
 		if(alsaerr == -1) snd_pcm_close(alsa_playback_handle);
@@ -368,23 +368,27 @@ int esd_audio_open()
 }
 
 #define ARCH_esd_audio_close
-void esd_audio_close()
+void esd_audio_close(void)
 {
 	if (alsadbg) {
 		fprintf(stderr, "esd_audio_close\n");
 		print_state();
 	}
 
-	if (alsa_playback_handle != NULL)
+	if (alsa_playback_handle != NULL) {
+		snd_pcm_hw_free( alsa_playback_handle );
 		snd_pcm_close( alsa_playback_handle );
-	if (alsa_capture_handle != NULL)
+	}
+	if (alsa_capture_handle != NULL) {
+		snd_pcm_hw_free( alsa_capture_handle );
 		snd_pcm_close(alsa_capture_handle);
+	}
 	alsa_playback_handle = NULL;
 	alsa_capture_handle = NULL;
 }
 
 #define ARCH_esd_audio_pause
-void esd_audio_pause()
+void esd_audio_pause(void)
 {
   
 	return;
@@ -444,49 +448,41 @@ int esd_audio_read( void *buffer, int buf_size )
 int esd_audio_write( void *buffer, int buf_size )
 {
 	int err;
-
 	int len = snd_pcm_bytes_to_frames(alsa_playback_handle, buf_size);
-	while ( ( err = snd_pcm_writei( alsa_playback_handle, buffer, len)) < 0) {
-		if (alsadbg) {
-			fprintf(stderr, "esd_audio_write\n");
-			print_state();
-		}
 
-		if (err == -EPIPE) {
-			if (alsadbg)
-				fprintf(stderr, "EPIPE\n");
-			if (( err = snd_pcm_prepare( alsa_playback_handle ) )< 0) {
+	while (len > 0) {
+		while ( ( err = snd_pcm_writei( alsa_playback_handle, buffer, len)) < 0) {
+			if (alsadbg) {
+				fprintf(stderr, "esd_audio_write\n");
+				print_state();
+			}
+
+			if (err == -EPIPE) {
+				if (alsadbg)
+					fprintf(stderr, "EPIPE\n");
+			} else  if (err == -ESTRPIPE) {
+				if (alsadbg)
+					fprintf(stderr, "ESTRPIPE\n");
+				while (( err = snd_pcm_resume(alsa_playback_handle)) == -EAGAIN)
+					sleep(1);
+			} 
+			err = snd_pcm_prepare(alsa_playback_handle) ;
+			if (err < 0) {
 				if (alsadbg)
 					fprintf(stderr, "%s\n", snd_strerror(err));
 				return -1;
 			}
 			continue;
-		} else  if (err == -ESTRPIPE) {
-			if (alsadbg)
-				fprintf(stderr, "ESTRPIPE\n");
-			while (( err = snd_pcm_resume(alsa_playback_handle)) == -EAGAIN)
-				sleep(1);
-			if (err < 0) {
-				if (alsadbg)
-					fprintf(stderr, "Preparing...\n");
-				if (snd_pcm_prepare( alsa_playback_handle) < 0)
-					return -1;
-			}
-			continue;
-		} 
-		err = snd_pcm_prepare(alsa_playback_handle) ;
-		if (err < 0) {
-			if (alsadbg)
-				fprintf(stderr, "%s\n", snd_strerror(err));
-			return -1;
 		}
+		len -= err;
+		buffer += snd_pcm_frames_to_bytes(alsa_playback_handle, err);
 	}
   
-	return ( snd_pcm_frames_to_bytes(alsa_playback_handle, err) );
+	return buf_size;
 }
 
 #define ARCH_esd_audio_flush
-void esd_audio_flush()
+void esd_audio_flush(void)
 {
   
 	if (alsadbg) {
@@ -494,8 +490,13 @@ void esd_audio_flush()
 		print_state();
 	}
 
+/* 2005-07-06, Martin Pitt <martin.pitt@ubuntu.com>:
+   Disable draining for now since it sometimes triggers a race
+   condition which makes the ALSA driver hang. */
+#if 0
 	if (alsa_playback_handle != NULL)
 		snd_pcm_drain( alsa_playback_handle );
+#endif
   
 	if (alsadbg) 
 		print_state();
