@@ -22,6 +22,11 @@
 #define gethostbyname2(host, family) gethostbyname((host))
 #endif /* HAVE_GETHOSTBYNAME2 */
 
+#ifndef SUN_LEN
+#define SUN_LEN(ptr) ((size_t) (((struct sockaddr_un *) 0)->sun_path)  \
+                     + strlen ((ptr)->sun_path))
+#endif
+
 /* AIX defines this */
 #ifndef h_errno
 extern int h_errno;
@@ -30,6 +35,34 @@ extern int h_errno;
 /* max arguments (argc + tokenized esd.conf) can't be more than this */
 #define MAX_OPTS 128
 
+typedef const char *(*audio_devices_t)(void);
+typedef int (*audio_open_t)(void);
+typedef void (*audio_close_t)(void);
+typedef void (*audio_pause_t)(void);
+typedef void (*audio_flush_t)(void);
+typedef int (*audio_write_t)( void *buffer, int buf_size );
+typedef int (*audio_read_t)( void *buffer, int buf_size );
+
+audio_devices_t impl_esd_audio_devices = esd_audio_devices;
+audio_open_t impl_esd_audio_open = esd_audio_open;
+audio_close_t impl_esd_audio_close = esd_audio_close;
+audio_pause_t impl_esd_audio_pause = esd_audio_pause;
+audio_flush_t impl_esd_audio_flush = esd_audio_flush;
+audio_write_t impl_esd_audio_write = esd_audio_write;
+audio_read_t impl_esd_audio_read = esd_audio_read;
+
+#define esd_audio_devices impl_esd_audio_devices
+#define esd_audio_open impl_esd_audio_open
+#define esd_audio_close impl_esd_audio_close
+#define esd_audio_pause impl_esd_audio_pause
+#define esd_audio_flush impl_esd_audio_flush
+#define esd_audio_write impl_esd_audio_write
+#define esd_audio_read impl_esd_audio_read
+
+#ifdef DRIVER_ARTS
+#include "audio_arts.c"
+#endif
+
 /*******************************************************************/
 /* esd.c - prototypes */
 void set_audio_buffer( void *buf, esd_format_t format, int magl, int magr,
@@ -341,10 +374,10 @@ int open_listen_socket(const char *hostname, int port )
     struct sockaddr_un socket_unix;
     int socket_listen = -1;
     struct linger lin;
-    size_t socket_len;
-    struct sockaddr *saddr;
+    size_t socket_len = 0;
+    struct sockaddr *saddr = NULL;
 
-	struct hostent *resolved;
+    struct hostent *resolved;
 
 
     /* create the socket, and set for non-blocking */
@@ -585,9 +618,12 @@ int main ( int argc, char *argv[] )
 
     /* from esd_config.c */
     extern char esd_spawn_options[];
+    extern char esd_default_options[];
+    extern int  esd_no_spawn;
+    int esd_spawning;
     
     /* for merging argv and esd.conf */
-    char tmp_str[LINEBUF_SIZE];
+    char tmp_str[2*LINEBUF_SIZE] = "";
     int num_opts = 0;
     char *opts[MAX_OPTS];
     char *tok;
@@ -607,11 +643,33 @@ int main ( int argc, char *argv[] )
 
     programname = *argv;
 
-    /* read in esd.conf, ~/.esd.conf or ESD_SPAWN_OPTIONS */
+#ifdef DRIVER_ARTS
+    artschk();
+#endif
+
+    /* read in esd.conf, ~/.esd.conf, ESD_SPAWN_OPTIONS or ESD_OPTIONS*/
     esd_config_read();
     
+    /* copy default options */
+    strncat (tmp_str, esd_default_options, LINEBUF_SIZE);
+
+
+    /* has server been spawned */
+    esd_spawning = 0;
+    if (!esd_no_spawn && (argc > 1)) {
+     for (i = 1; i < argc - 1 ; i++) {
+	    if (argv[i] && strcmp("-spawnfd", argv[i]) == 0) {
+		esd_spawning = 1;
+		break;		
+	    }
+      }
+    }
+	    
+    
     /* copy esd_spawn_options to tmp_str because of strtok */
-    strncpy(tmp_str, esd_spawn_options, LINEBUF_SIZE);
+    if (esd_spawning) {
+      strncpy(tmp_str, esd_spawn_options, LINEBUF_SIZE);
+    }
     
     /* add esd.conf options */
     num_opts = 0;
@@ -700,6 +758,8 @@ int main ( int argc, char *argv[] )
 #endif
 	} else if ( !strcmp( opts[ arg ], "-nobeeps" ) ) {
 	    esd_beeps = 0;
+	} else if ( !strcmp( opts[ arg ], "-beeps" ) ) {
+	    esd_beeps = 1;
 	} else if ( !strcmp( opts[ arg ], "-unix" ) ) {
 	    esd_use_tcpip = 0;
 	} else if ( !strcmp( opts[ arg ], "-tcp" ) ) {
@@ -711,6 +771,8 @@ int main ( int argc, char *argv[] )
 	    esd_is_locked = 0;
 	} else if ( !strcmp( opts[ arg ], "-terminate" ) ) {
 	    esd_terminate = 1;
+	} else if ( !strcmp( opts[ arg ], "-noterminate" ) ) {
+	    esd_terminate = 0;
 	} else if ( !strcmp( opts[ arg ], "-spawnpid" ) ) {
 	    if ( ++arg < num_opts )
 		esd_spawnpid = atoi( opts[ arg ] );
@@ -729,13 +791,15 @@ int main ( int argc, char *argv[] )
 	    fprintf( stderr, "  -d DEVICE     force esd to use sound device DEVICE\n" );
 	    fprintf( stderr, "  -b            run server in 8 bit sound mode\n" );
 	    fprintf( stderr, "  -r RATE       run server at sample rate of RATE\n" );
-	    fprintf( stderr, "  -as SECS      free audio device after SECS of inactivity\n" );
+	    fprintf( stderr, "  -as SECS      free audio device after SECS of inactivity (-1 to disable)\n" );
 	    fprintf( stderr, "  -unix         use unix domain sockets instead of tcp/ip\n" );
 	    fprintf( stderr, "  -tcp          use tcp/ip sockets instead of unix domain\n" );
 	    fprintf( stderr, "  -public       make tcp/ip access public (other than localhost)\n" );
 	    fprintf( stderr, "  -promiscuous  start unlocked and owned (disable authenticaton) NOT RECOMMENDED\n" );
 	    fprintf( stderr, "  -terminate    terminate esd daemon after last client exits\n" );
+	    fprintf( stderr, "  -noterminate  do not terminate esd daemon after last client exits\n" );
 	    fprintf( stderr, "  -nobeeps      disable startup beeps\n" );
+	    fprintf( stderr, "  -beeps        enable startup beeps\n" );
 	    fprintf( stderr, "  -trust        start esd even if use of %s can be insecure\n",
 		     ESD_UNIX_SOCKET_DIR );
 #ifdef ESDBG
