@@ -13,6 +13,8 @@ int esd_spawn_wait_ms=100; /* Time to wait trying to connect to an
 			      autospawned ESD, in milliseconds. */
 char esd_spawn_options[LINEBUF_SIZE] = "-terminate -nobeeps -as 2";
 
+char esd_default_options [LINEBUF_SIZE] = ""; /* Default options, always applied */
+
 static int read_esd_config = 0;
 
 static void esd_config_read_file(FILE *fh);
@@ -21,7 +23,8 @@ void
 esd_config_read(void)
 {
   FILE *fh;
-  char *fn, *tmpenv;
+  char *fn;
+  const char *tmpenv;
 
   if(read_esd_config) return;
 
@@ -59,6 +62,10 @@ esd_config_read(void)
   if(tmpenv && strlen(tmpenv) < (sizeof(esd_spawn_options) - 1))
     strcpy(esd_spawn_options, tmpenv);
 
+  tmpenv = getenv("ESD_DEFAULT_OPTIONS");
+  if(tmpenv && strlen(tmpenv) < (sizeof(esd_default_options) - 1))
+    strcpy(esd_default_options, tmpenv);
+
   read_esd_config = 1;
 }
 
@@ -113,6 +120,10 @@ esd_config_read_file(FILE *fh)
 	{
 	  strcpy(esd_spawn_options, value);
 	}
+      else if(!strcasecmp(key, "default_options"))
+	{
+	  strcpy(esd_default_options, value);
+	}
       else if(!strcasecmp(key, "spawn_wait_ms"))
 	{
 	  char *endptr;
