@@ -26,6 +26,8 @@
 
 #if defined(__GNUC__) && !defined(__STRICT_ANSI__)
 
+#define _GNU_SOURCE 1
+
 #ifdef DSP_DEBUG
 #define DPRINTF(format, args...)	printf(format, ## args)
 #else
@@ -42,6 +44,7 @@
 #include <fcntl.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/time.h>
 #include <stdio.h>
 
 #ifdef HAVE_MACHINE_SOUNDCARD_H
@@ -69,10 +72,13 @@ typedef unsigned long request_t;
 typedef int request_t;
 #endif
 
-static int sndfd = -1, mixfd = -1;
-static int settings = 0, done = 0;
+static int sndfd = -1;
+static int mixfd = -1;
+static int settings = 0;
+static int done = 0;
 
-static char *ident = NULL, *mixer = NULL;
+static char *ident = NULL;
+static char *mixer = NULL;
 static int use_mixer = 0;
 
 /*
@@ -80,7 +86,7 @@ static int use_mixer = 0;
  */
 static int mmapemu = 0;
 static count_info mmapemu_ocount;
-static char *mmapemu_obuffer = 0;
+static char *mmapemu_obuffer = NULL;
 static int mmapemu_osize;
 static long long mmapemu_bytes_per_sec;
 struct timeval mmapemu_last_flush;
@@ -127,9 +133,12 @@ dsp_init (void)
 {
   if (!ident)
     {
-      char *str = getenv ("ESDDSP_NAME");
-      ident = malloc (ESD_NAME_MAX);
+      const char *str;
+     
+      str = getenv ("ESDDSP_NAME");
+      ident = malloc (ESD_NAME_MAX+1);
       strncpy (ident, (str ? str : "esddsp"), ESD_NAME_MAX);
+      ident[ESD_NAME_MAX] = '\0';
 
       str = getenv("ESDDSP_MMAP");
       mmapemu = str && !strcmp(str,"1");
@@ -182,7 +191,7 @@ mix_init (int *esd, int *player)
     }
 }
 
-static void mmapemu_flush()
+static void mmapemu_flush(void)
 {
   struct timeval current_time;
   long long usec_since_last_flush;
@@ -211,24 +220,29 @@ static void mmapemu_flush()
     }
 }
 
+typedef int (*open_funcptr_t) (const char *, int, mode_t);
+
+/* The wrap_* functions act as wrapper for both the 32-bit versions of
+ * the functions (open/mmap/fopen), as well as for the 64-bit ones
+ * (open64/mmap64/fopen64). Just to avoid massive code duplication.
+ *   -- Sebastian Schmidt <yath@yath.de>, 2005-12-15
+ */
 int
-open (const char *pathname, int flags, ...)
+wrap_open (const char *symname, open_funcptr_t *func, const char *pathname,
+                int flags, mode_t mode)
 {
-  static int (*func) (const char *, int, mode_t) = NULL;
-  va_list args;
-  mode_t mode;
-
-  if (!func)
-    func = (int (*) (const char *, int, mode_t)) dlsym (REAL_LIBC, "open");
-
-  dsp_init ();
-
-  va_start (args, flags);
-  mode = va_arg (args, mode_t);
-  va_end (args);
+  if (!*func) {
+    *func = (open_funcptr_t) dlsym (REAL_LIBC, symname);
+    dsp_init ();
+  }
 
   if (!strcmp (pathname, "/dev/dsp"))
     {
+/* We shouldn't need this with Unix domain sockets, plus there
+ * is a better test in esdlib.c - for more info see:
+ * http://www.debian.org/Bugs/db/53/53602.html
+ * commented out by bma@debian.org 1999-12-29 */
+/*
       if (!getenv ("ESPEAKER") && !mmapemu)
 	{
           int ret;
@@ -237,20 +251,55 @@ open (const char *pathname, int flags, ...)
 	  if ((ret = (*func) (pathname, flags, mode)) >= 0)
 	    return ret;
 	}
+*/
 
-      DPRINTF ("hijacking /dev/dsp open, and taking it to esd...\n");
+      DPRINTF ("hijacking /dev/dsp %s, and taking it to esd...\n", symname);
       settings = done = 0;
       return (sndfd = esd_open_sound (NULL));
     }
   else if (use_mixer && !strcmp (pathname, "/dev/mixer"))
     {
-      DPRINTF ("hijacking /dev/mixer open, and taking it to esd...\n");
-      return (mixfd = (*func) (mixer, O_RDWR | O_CREAT, S_IRUSR | S_IWUSR));
+      DPRINTF ("hijacking /dev/mixer %s, and taking it to esd...\n", symname);
+      return (mixfd = (**func) (mixer, O_RDWR, S_IRUSR | S_IWUSR));
     }
   else
-    return (*func) (pathname, flags, mode);
+    return (**func) (pathname, flags, mode);
 }
 
+int
+open (const char *pathname, int flags, ...)
+{
+  static open_funcptr_t func = NULL;
+  mode_t mode = 0;
+  va_list args;
+  
+  if (flags & O_CREAT) {
+    va_start(args, flags);
+    mode = va_arg(args, mode_t);
+    va_end(args);
+  }
+
+  return wrap_open("open", &func, pathname, flags, mode);
+}
+
+#ifdef HAVE_OPEN64
+int
+open64 (const char *pathname, int flags, ...)
+{
+  static open_funcptr_t func = NULL;
+  mode_t mode = 0;
+  va_list args;
+  
+  if (flags & O_CREAT) {
+    va_start(args, flags);
+    mode = va_arg(args, mode_t);
+    va_end(args);
+  }
+
+  return wrap_open("open64", &func, pathname, flags, mode);
+}
+#endif
+
 static int
 dspctl (int fd, request_t request, void *argp)
 {
@@ -284,7 +333,11 @@ dspctl (int fd, request_t request, void *argp)
       break;
 
     case SNDCTL_DSP_GETBLKSIZE:
-      *arg = ESD_BUF_SIZE;
+      if (mmapemu)
+      	*arg = MMAPEMU_FRAGSIZE;
+      else
+      	*arg = ESD_BUF_SIZE;
+     
       break;
 
     case SNDCTL_DSP_GETFMTS:
@@ -306,14 +359,25 @@ dspctl (int fd, request_t request, void *argp)
 	audio_buf_info *bufinfo = (audio_buf_info *) argp;
 	if (mmapemu)
 	  {
+	    /* Number of fragments allocated for buffering */
 	    bufinfo->fragstotal = MMAPEMU_FRAGSTOTAL;
+	    /* Size of a fragment in bytes (same as SNDCTL_DSP_GETBLKSIZE) */
 	    bufinfo->fragsize = MMAPEMU_FRAGSIZE;
-	    bufinfo->bytes = 0;              /* FIXME: is this right? */
-	    bufinfo->fragments = 0;
+	    /* Number of bytes that can be read or written immediately without blocking */
+	    bufinfo->bytes = MMAPEMU_FRAGSIZE;
+	    /* Number of full fragments that can be read or written without blocking */
+	    bufinfo->fragments = 1;
 	  }
 	else
 	  {
+	    /* Number of fragments allocated for buffering */
+	    bufinfo->fragstotal = 1;
+	    /* Size of a fragment in bytes (same as SNDCTL_DSP_GETBLKSIZE) */
+	    bufinfo->fragsize = ESD_BUF_SIZE;
+	    /* Number of bytes that can be read or written immediately without blocking */
 	    bufinfo->bytes = ESD_BUF_SIZE;
+	    /* Number of full fragments that can be read or written without blocking */
+	    bufinfo->fragments = 1;
 	  }
       }
       break;
@@ -343,13 +407,13 @@ dspctl (int fd, request_t request, void *argp)
 
       done = 1;
 
-      if (write (sndfd, &proto, sizeof (proto)) != sizeof (proto))
+      if (write (fd, &proto, sizeof (proto)) != sizeof (proto))
         return -1;
-      if (write (sndfd, &fmt, sizeof (fmt)) != sizeof (fmt))
+      if (write (fd, &fmt, sizeof (fmt)) != sizeof (fmt))
         return -1;
-      if (write (sndfd, &speed, sizeof (speed)) != sizeof (speed))
+      if (write (fd, &speed, sizeof (speed)) != sizeof (speed))
         return -1;
-      if (write (sndfd, ident, ESD_NAME_MAX) != ESD_NAME_MAX)
+      if (write (fd, ident, ESD_NAME_MAX) != ESD_NAME_MAX)
         return -1;
 
       if (mmapemu)
@@ -380,7 +444,7 @@ dspctl (int fd, request_t request, void *argp)
   return 0;
 }
 
-int
+static int
 mixctl (int fd, request_t request, void *argp)
 {
   static int esd = -1, player = -1;
@@ -494,20 +558,21 @@ close (int fd)
   return (*func) (fd);
 }
 
+typedef void *(*mmap_funcptr_t) (void *, size_t, int, int, int, off_t);
+
 void *
-mmap (void  *start,  size_t length, int prot, int flags, int fd, off_t offset)
+wrap_mmap (const char *symname, mmap_funcptr_t *func, void *start, 
+                size_t length, int prot, int flags, int fd, off_t offset)
 {
-  static void *(*func) (void *, size_t, int, int, int, off_t) = NULL;
-  
-  if (!func)
-    func = (void * (*) (void *, size_t, int, int, int, off_t)) dlsym (REAL_LIBC, "mmap");
+  if (!*func)
+    *func = (mmap_funcptr_t) dlsym (REAL_LIBC, symname);
 
   if(fd != sndfd || sndfd == -1)
-    return (*func)(start,length,prot,flags,fd,offset);
+    return (**func)(start,length,prot,flags,fd,offset);
   else
     {
-      DPRINTF ("esddsp: mmap - start = %x, length = %d, prot = %d\n",
-	       start, length, prot);
+      DPRINTF ("esddsp: %s - start = %x, length = %d, prot = %d\n",
+	       symname, start, length, prot);
       DPRINTF ("      flags = %d, fd = %d, offset = %d\n",flags, fd,offset);
       if(mmapemu)
 	{
@@ -516,11 +581,27 @@ mmap (void  *start,  size_t length, int prot, int flags, int fd, off_t offset)
 	  mmapemu_ocount.ptr = mmapemu_ocount.blocks = mmapemu_ocount.bytes = 0;
 	  return mmapemu_obuffer;
 	}
-      else DPRINTF ("esddsp: /dev/dsp mmap (unsupported, try -m option)...\n");
+      else DPRINTF ("esddsp: /dev/dsp %s (unsupported, try -m option)...\n", symname);
     }
   return (void *)-1;
 }
 
+void *
+mmap (void *start, size_t length, int prot, int flags, int fd, off_t offset)
+{
+  static mmap_funcptr_t func = NULL;
+  return wrap_mmap("mmap", &func, start, length, prot, flags, fd, offset);
+}
+
+#ifdef HAVE_MMAP64
+void *
+mmap64 (void *start, size_t length, int prot, int flags, int fd, off_t offset)
+{
+  static mmap_funcptr_t func = NULL;
+  return wrap_mmap("mmap64", &func, start, length, prot, flags, fd, offset);
+}
+#endif
+
 int
 munmap (void *start, size_t length)
 {
@@ -533,7 +614,7 @@ munmap (void *start, size_t length)
     return (*func)(start,length);
 
   DPRINTF ("esddsp: /dev/dsp munmap...\n");
-  mmapemu_obuffer = 0;
+  mmapemu_obuffer = NULL;
   free(start);
 
   return 0;
@@ -551,7 +632,7 @@ int
 unlink (const char *filename)
 {
   static int (*func) (const char *) = NULL;
-  char *num;
+  const char *num;
 
   if (!func)
     func = (int (*) (const char *)) dlsym (REAL_LIBC, "unlink");
@@ -571,7 +652,7 @@ typedef int (*sa_func_t) (int, struct sockaddr *, int);
 static int
 sockaddr_mangle (sa_func_t func, int fd, struct sockaddr *addr, int len)
 {
-  char *num;
+  const char *num;
 
   if (!strcmp (((struct sockaddr_un *) addr)->sun_path, "/tmp/X11Amp_CTRL")
       && (num = getenv(ENVSET)))
@@ -616,6 +697,54 @@ connect (int fd, struct sockaddr *addr, int len)
 
 #endif /* MULTIPLE_X11AMP */
 
+#if ESDDSP_STDIO_EMU
+/*
+ * use #include rather than linking, because we want to keep everything
+ * static inside esddsp to be sure that we don't override application symbols
+ */
+#include "stdioemu.c"
+
+typdef FILE *(*fopen_funcptr_t) (const char *, const char *);
+
+FILE *
+wrap_fopen (const char *symname, fopen_funcptr_t *func, const char *pathname,
+                const char *mode)
+{
+  static FILE *(*func) (const char *, const char *) = NULL;
+
+  if (!*func) {
+    *func = (fopen_funcptr_t) dlsym (REAL_LIBC, symname);
+    dsp_init ();
+  }
+
+  if (!strcmp (pathname, "/dev/dsp"))
+    {
+      return fake_fopen(pathname,mode);
+    }
+  else if (use_mixer && !strcmp (pathname, "/dev/mixer"))
+    {
+      return fake_fopen(pathname,mode);
+    }
+  else
+    return (**func) (pathname, mode);
+}
+FILE *
+fopen(const char *pathname, const char *mode)
+{
+  static fopen_funcptr_t func;
+  return wrap_fopen("fopen", &func, pathname, mode);
+}
+
+#ifdef HAVE_FOPEN64
+FILE *
+fopen64(const char *pathname, const char *mode)
+{
+  static fopen_funcptr_t func;
+  return wrap_fopen("fopen64", &func, pathname, mode);
+}
+#endif
+#endif
+
 #else /* __GNUC__ */
 
 void
@@ -625,4 +754,3 @@ nogcc (void)
 }
 
 #endif
-
