@@ -23,6 +23,11 @@
 
 #include <sys/un.h>
 
+#ifndef SUN_LEN
+#define SUN_LEN(ptr) ((size_t) (((struct sockaddr_un *) 0)->sun_path)  \
+                     + strlen ((ptr)->sun_path))
+#endif
+
 /*******************************************************************/
 /* prototypes */
 int esd_set_socket_buffers( int sock, int src_format, 
@@ -158,9 +163,9 @@ int esd_send_auth( int sock )
     int auth_fd = -1;
     int endian = ESD_ENDIAN_KEY;
     int reply;
-    char *auth_filename = 0;
+    char *auth_filename = NULL;
     unsigned char auth_key[ESD_KEY_LEN];
-    char *home = NULL;
+    const char *home = NULL;
     int namelen, retval;
     void (*phandler)(int);
   
@@ -437,26 +442,36 @@ esd_connect_tcpip(const char *host)
        connect_host[0] = '\0';   
 
        if ( host ) {
+         char *host_dup;
+         char *s;
+
+         host_dup = strdup (host);
+         if (!host_dup) {
+           fprintf(stderr, "Out of memory\n");
+           return -1;
+         }
+         s = host_dup;
          cnt = 0;
-         for ( i = 0; i < strlen (host); i++ )
-           if ( host[i] == ':' )
+         for ( i = 0; i < strlen (s); i++ )
+           if ( s[i] == ':' )
              cnt++;
 
          if ( cnt == 1 ) {
-           loc = strchr (host, ':');
+           loc = strchr (s, ':');
            *loc = '\0';
-           strcpy (connect_host, host);
+           strcpy (connect_host, s);
            port = atoi (loc + 1);
          }
          else {
-           if (( cnt != 0 ) && ((loc = strchr (host, ']')) != NULL )) {
+           if (( cnt != 0 ) && ((loc = strchr (s, ']')) != NULL )) {
              *loc = '\0';
-             strcpy ( connect_host, ++host );
+             strcpy ( connect_host, ++s );
              port = atoi ( loc + 2 );
            }
            else 
-             strcpy ( connect_host, host );
+             strcpy ( connect_host, s );
          }
+         free (host_dup);
        }
        if( port == 0 )
          port = ESD_DEFAULT_PORT;
@@ -605,7 +620,7 @@ esd_connect_tcpip(const char *host)
  * Return Value: -1 on error, else a socket number connected to ESD.
  */
 static int
-esd_connect_unix()
+esd_connect_unix(void)
 {
     struct sockaddr_un socket_unix;
     int socket_out = -1;
@@ -689,7 +704,7 @@ int esd_open_sound( const char *host )
     int len;
     char use_unix = 0;
     char display_host[ 256 ];
-    char *display;
+    const char *display;
 
     if ( !host ) host = getenv("ESPEAKER");
 
@@ -700,7 +715,7 @@ int esd_open_sound( const char *host )
 	   and see if we strike gold */
 	len = strcspn( display, ":" );
 	if ( len ) {
-	    len = min( len, 256 ); 
+	    len = min( len, 255 ); 
 	    strncpy( display_host, display, len );
 	    display_host[ len ] = '\0';
 	    host = display_host;
@@ -714,7 +729,7 @@ int esd_open_sound( const char *host )
 	    use_unix = 1;
     }
     if ( use_unix )
-	socket_out = esd_connect_unix( NULL );
+	socket_out = esd_connect_unix();
     if ( socket_out >= 0 ) goto finish_connect;
 
     socket_out = esd_connect_tcpip( host );
@@ -784,7 +799,7 @@ int esd_open_sound( const char *host )
 	    ret = read (esd_pipe[0], &c, 1);
 
 	    if (ret == 1) {
-		socket_out = esd_connect_unix(host);
+		socket_out = esd_connect_unix();
 		if (socket_out < 0)
 		    socket_out = esd_connect_tcpip(host);
 	    }
